import type { DataTransformer } from '../types';
/** ISOCalendarDateTime Data Transformer */
declare class ISOCalendarDateTimeDataTransformer implements DataTransformer {
    /** Database time format is in milliseconds */
    private readonly isDbInMs;
    /** ISOCalendarDateDataTransformer instance to use for transformations of the date part of the DateTime */
    private readonly isoCalendarDateTransformer;
    /** ISOTimeDataTransformer instance to use for transformations of the time part of the DateTime */
    private readonly isoTimeTransformer;
    constructor(dbFormat?: 's' | 'ms');
    /** Transform mv style timestamp data (ddddd.sssss[SSS]) to ISO 8601 approved date/time format (yyyy-mm-ddTHH:mm:ss.SSS) */
    transformFromDb(value: null): null;
    transformFromDb(value: unknown): string;
    /**
     * Transform ISO 8601 approved date/time format (yyyy-mm-ddTHH:mm:ss.SSS) to mv style timestamp data (ddddd.sssss[SSS])
     * @throws {@link TransformDataError} Value could not be transformed to database format
     */
    transformToDb(value: null): null;
    transformToDb(value: unknown): string;
    /** Transform query constants to internal u2 date-time */
    transformToQuery(value: unknown): string;
}
export default ISOCalendarDateTimeDataTransformer;
