import type { DataTransformer } from '../types';
/** ISOTime Data Transformer */
declare class ISOTimeDataTransformer implements DataTransformer {
    /** Database time format is in milliseconds */
    private readonly isDbInMs;
    constructor(dbFormat?: 's' | 'ms');
    /**
     * Transform mv style time data to ISO 8601 approved time format (HH:mm:ss.SSS)
     * @throws {@link TransformDataError} Database value could not be transformed to external format
     */
    transformFromDb(value: null): null;
    transformFromDb(value: unknown): string;
    /**
     * Transform ISO 8601 approved time format (HH:mm:ss.SSS) to mv style time data
     * @throws {@link TransformDataError} Value could not be transformed to database format
     */
    transformToDb(value: null): null;
    transformToDb(value: unknown): string;
    /** Transform query constants to internal u2 time format */
    transformToQuery(value: unknown): string;
    /** Convert internal time to ISOTime format */
    private convertInternalTimeToISOTime;
    /** Convert hours, minutes, seconds, and milliseconds to multivalue internal time format */
    private convertISOTimeToInternalTime;
}
export default ISOTimeDataTransformer;
