import type { DataTransformer } from '../types';
/**
 * Number Data Transformer
 * @throws {@link InvalidParameterError} An invalid parameter was passed to the function
 */
declare class NumberDataTransformer implements DataTransformer {
    /** Number of implied decimals in database storage */
    private readonly dbDecimals;
    constructor(dbDecimals?: number);
    /**
     * Transform mv style internally formatted numeric data (nnnnn) to externally formatted numeric data (nnn.nn)
     * @throws {@link TransformDataError} Database value could not be transformed to external format
     */
    transformFromDb(value: null): null;
    transformFromDb(value: unknown): number;
    /** Transform externally formatted numeric data (nnn.nn) to mv style internally formatted numeric data */
    transformToDb(value: null): null;
    transformToDb(value: unknown): string;
    /** Transform query constants to u2 formatted string number */
    transformToQuery(value: unknown): string;
}
export default NumberDataTransformer;
